package com.ethosa.ktc.utils

import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.drawable.BitmapDrawable
import android.graphics.drawable.Drawable
import android.text.Html
import android.widget.TextView
import com.squareup.picasso.Picasso
import kotlinx.coroutines.*

/**
 * Provides image fetching from HTML text
 */
class HtmlImageGetter(
    private val res: Resources,
    private val textView: TextView
) : Html.ImageGetter {
    /**
     * Fetches images from HTML text
     */
    @Suppress("EXPERIMENTAL_IS_NOT_ENABLED")
    @OptIn(DelicateCoroutinesApi::class)
    override fun getDrawable(p0: String?): Drawable {
        val holder = PlaceHolder(res, null)
        GlobalScope.launch(Dispatchers.IO) {
            run {
                // Download image and converts to Drawable
                val bitmap = Picasso.get().load(p0!!).get()
                val drawable = BitmapDrawable(res, bitmap)
                // Scale image with keep aspect ratio
                val width = res.displayMetrics.widthPixels
                val aspectRatio =
                    drawable.intrinsicWidth.toFloat() / drawable.intrinsicHeight.toFloat()
                val height = (width / aspectRatio).toInt()
                // Change bounds
                drawable.setBounds(0, 0, width, height)
                holder.drawable = drawable
                holder.setBounds(0, 0, width, height)
                // Update text
                withContext(Dispatchers.Main) {
                    textView.text = textView.text
                }
            }
        }
        return holder
    }

    /**
     * Drawable placeholder
     */
    class PlaceHolder(
        res: Resources,
        bitmap: Bitmap?
    ) : BitmapDrawable(res, bitmap) {
        var drawable: Drawable? = null
        /**
         * Draws canvas on drawable.
         */
        override fun draw(canvas: Canvas) {
            drawable?.run { draw(canvas) }
        }
    }
}